@@ -355,14 +355,15 @@ def func(parser, options, args):
         out.start('Cloning current branch to "%s"' % clone_name)
 
         if stack:
+            check_head_top_equal(stack)
             clone = stack.clone(clone_name, 'branch clone of %s' % cur_branch.name)
-            trans = StackTransaction(clone, 'clone')
+            trans = StackTransaction(clone)
             try:
                 for pn in stack.patchorder.applied:
                     trans.push_patch(pn)
             except TransactionHalted:
                 pass
-            trans.run()
+            trans.execute('clone')
         else:
             clone = Stack.create(
                 repository,
