@@ -1,5 +1,5 @@
 from stgit.argparse import opt
-from stgit.commands.common import DirectoryHasRepository
+from stgit.commands.common import DirectoryHasRepository, check_head_top_equal
 from stgit.lib import transaction
 
 __copyright__ = """
@@ -46,7 +46,8 @@ directory = DirectoryHasRepository()
 
 
 def _clean(stack, clean_applied, clean_unapplied):
-    trans = transaction.StackTransaction(stack, 'clean', allow_conflicts=True)
+    check_head_top_equal(stack)
+    trans = transaction.StackTransaction(stack, allow_conflicts=True)
 
     def del_patch(pn):
         if pn in stack.patchorder.applied:
@@ -63,7 +64,7 @@ def _clean(stack, clean_applied, clean_unapplied):
 
     for pn in trans.delete_patches(del_patch):
         trans.push_patch(pn)
-    trans.run()
+    trans.execute('clean')
 
 
 def func(parser, options, args):
