@@ -129,6 +129,14 @@ def check_conflicts(iw):
         )
 
 
+def check_index_and_worktree_clean(stack):
+    iw = stack.repository.default_iw
+    if not iw.worktree_clean():
+        raise CmdException('Worktree not clean. Use "refresh" or "reset --hard"')
+    if not iw.index.is_clean(stack.head):
+        raise CmdException('Index not clean. Use "refresh" or "reset --hard"')
+
+
 def print_current_patch(stack):
     if stack.patchorder.applied:
         out.info('Now at patch "%s"' % stack.patchorder.applied[-1])
@@ -263,8 +271,10 @@ def apply_patch(stack, diff, base=None, reject=False, strip=None, context_lines=
 
 def prepare_rebase(stack, cmd_name):
     # pop all patches
+    check_head_top_equal(stack)
+    check_index_and_worktree_clean(stack)
     iw = stack.repository.default_iw
-    trans = StackTransaction(stack, '%s (pop)' % cmd_name, check_clean_iw=iw)
+    trans = StackTransaction(stack)
     out.start('Popping all applied patches')
     try:
         trans.reorder_patches(
@@ -275,7 +285,7 @@ def prepare_rebase(stack, cmd_name):
         )
     except TransactionException:
         pass
-    retval = trans.run(iw, print_current_patch=False)
+    retval = trans.execute('%s (pop)' % cmd_name, iw, print_current_patch=False)
     if retval:
         out.done('Failed to pop applied patches')
     else:
@@ -307,7 +317,8 @@ def rebase(stack, iw, target_commit=None):
 
 def post_rebase(stack, applied, cmd_name, check_merged):
     iw = stack.repository.default_iw
-    trans = StackTransaction(stack, '%s (reapply)' % cmd_name)
+    check_head_top_equal(stack)
+    trans = StackTransaction(stack)
     try:
         if check_merged:
             merged = set(trans.check_merged(applied))
@@ -319,7 +330,25 @@ def post_rebase(stack, applied, cmd_name, check_merged):
             )
     except TransactionHalted:
         pass
-    return trans.run(iw)
+    return trans.execute('%s (reapply)' % cmd_name, iw)
+
+
+def hide_patches(stack, iw, patches):
+    def allow_conflicts(trans):
+        # Allow conflicts if the topmost patch stays the same.
+        if stack.patchorder.applied:
+            return trans.applied and trans.applied[-1] == stack.patchorder.applied[-1]
+        else:
+            return not trans.applied
+
+    trans = StackTransaction(stack, 'hide', allow_conflicts=allow_conflicts)
+    try:
+        to_push = trans.hide_patches(lambda pn: pn in patches)
+        for pn in to_push:
+            trans.push_patch(pn, iw)
+    except TransactionHalted:
+        pass
+    return trans.execute('hide', iw)
 
 
 def delete_patches(stack, iw, patches):
@@ -330,14 +359,15 @@ def delete_patches(stack, iw, patches):
         else:
             return not trans.applied
 
-    trans = StackTransaction(stack, 'delete', allow_conflicts=allow_conflicts)
+    check_head_top_equal(stack)
+    trans = StackTransaction(stack, allow_conflicts=allow_conflicts)
     try:
         to_push = trans.delete_patches(lambda pn: pn in patches)
         for pn in to_push:
             trans.push_patch(pn, iw)
     except TransactionHalted:
         pass
-    return trans.run(iw)
+    return trans.execute('delete', iw)
 
 
 #
