@@ -1,7 +1,5 @@
 """Patch editing command"""
 
-import io
-
 from stgit import argparse, utils
 from stgit.argparse import opt
 from stgit.commands.common import (
@@ -10,8 +8,7 @@ from stgit.commands.common import (
     run_commit_msg_hook,
 )
 from stgit.config import config
-from stgit.lib import edit, transaction
-from stgit.out import out
+from stgit.lib import edit
 
 __copyright__ = """
 Copyright (C) 2007, Catalin Marinas <catalin.marinas@gmail.com>
@@ -39,17 +36,21 @@ diff.
 
 The editor is invoked with the following contents:
 
-  From: A U Thor <author@example.com>
-  Date: creation date
+....
+From: A U Thor <author@example.com>
+Date: creation date
 
-  Patch description
+Patch description
+....
 
 If --diff was specified, the diff appears at the bottom, after a
 separator:
 
-  ---
+....
+---
 
-  Diff text
+Diff text
+....
 
 Command-line options can be used to modify specific information
 without invoking the editor. (With the --edit option, the editor is
@@ -67,27 +68,26 @@ built on top of StGit, such as the Emacs mode. See also the --set-tree
 flag of stg push."""
 
 args = ['applied_patches', 'unapplied_patches', 'hidden_patches']
-options = (
-    [
-        opt('-d', '--diff', action='store_true', short='Edit the patch diff'),
-        opt('-e', '--edit', action='store_true', short='Invoke interactive editor'),
-    ]
-    + argparse.sign_options()
-    + argparse.message_options(save_template=True)
-    + argparse.hook_options()
-    + argparse.author_options()
-    + argparse.diff_opts_option()
-    + [
-        opt(
-            '-t',
-            '--set-tree',
-            action='store',
-            metavar='TREE-ISH',
-            short='Set the git tree of the patch to TREE-ISH',
-        )
-    ]
+options = [
+    opt('-d', '--diff', action='store_true', short='Edit the patch diff'),
+    opt('-e', '--edit', action='store_true', short='Invoke interactive editor'),
+]
+options.extend(argparse.trailer_options())
+options.extend(argparse.message_options(save_template=True))
+options.extend(argparse.hook_options())
+options.extend(argparse.author_options())
+options.extend(argparse.diff_opts_option())
+options.append(
+    opt(
+        '-t',
+        '--set-tree',
+        action='store',
+        metavar='TREE-ISH',
+        short='Set the git tree of the patch to TREE-ISH',
+    )
 )
 
+
 directory = DirectoryHasRepository()
 
 
@@ -97,7 +97,7 @@ def func(parser, options, args):
 
     if len(args) == 0:
         if not stack.patchorder.applied:
-            raise CmdException('Cannot edit top patch, because no patches are applied')
+            raise CmdException('Cannot edit top patch because no patches are applied')
         patchname = stack.patchorder.applied[-1]
     elif len(args) == 1:
         [patchname] = args
@@ -124,85 +124,49 @@ def func(parser, options, args):
             else options.message.encode(config.get('i18n.commitencoding'))
         ),
         author=options.author,
-        sign_str=options.sign_str,
+        trailers=options.trailers,
     )
 
     if options.save_template:
         options.save_template(
-            edit.patch_desc(
-                stack.repository,
-                cd,
-                options.diff,
-                options.diff_flags,
-                replacement_diff=None,
+            edit.get_patch_description(
+                stack.repository, cd, patchname, options.diff, options.diff_flags
             )
         )
         return utils.STGIT_SUCCESS
 
     use_editor = cd == orig_cd or options.edit or options.diff
     if use_editor:
-        cd, failed_diff = edit.interactive_edit_patch(
-            stack.repository, cd, options.diff, options.diff_flags
+        cd, new_patchname, failed_diff = edit.interactive_edit_patch(
+            stack.repository, cd, patchname, options.diff, options.diff_flags
         )
+        # If we couldn't apply the patch, fail without even trying to
+        # affect any of the changes.
+        if failed_diff is not None:
+            return utils.STGIT_COMMAND_ERROR
     else:
-        failed_diff = None
-
-    def failed(reason='Edited patch did not apply.'):
-        fn = '.stgit-failed.patch'
-        with io.open(fn, 'wb') as f:
-            f.write(
-                edit.patch_desc(
-                    stack.repository,
-                    cd,
-                    options.diff,
-                    options.diff_flags,
-                    replacement_diff=failed_diff,
-                )
-            )
-        out.error(reason, 'The patch has been saved to "%s".' % fn)
-        return utils.STGIT_COMMAND_ERROR
-
-    # If we couldn't apply the patch, fail without even trying to
-    # effect any of the changes.
-    if failed_diff:
-        return failed()
+        new_patchname = None
 
     if not options.no_verify and (use_editor or cd.message != orig_cd.message):
         try:
             cd = run_commit_msg_hook(stack.repository, cd, use_editor)
         except Exception:
             if options.diff:
-                failed('The commit-msg hook failed.')
+                patch_desc = edit.get_patch_description(
+                    stack.repository, cd, patchname, options.diff, options.diff_flags
+                )
+                edit.note_patch_application_failure(
+                    patch_desc, 'The commit-msg hook failed.'
+                )
             raise
 
-    # Refresh the committer information
-    cd = cd.set_committer(None)
-
-    # The patch applied, so now we have to rewrite the StGit patch
-    # (and any patches on top of it).
-    iw = stack.repository.default_iw
-    trans = transaction.StackTransaction(stack, 'edit', allow_conflicts=True)
-    if patchname in trans.applied:
-        popped = trans.applied[trans.applied.index(patchname) + 1 :]
-        popped_extra = trans.pop_patches(lambda pn: pn in popped)
-        assert not popped_extra
-    else:
-        popped = []
-    trans.patches[patchname] = stack.repository.commit(cd)
-    try:
-        for pn in popped:
-            if options.set_tree:
-                trans.push_tree(pn)
-            else:
-                trans.push_patch(pn, iw, allow_interactive=True)
-    except transaction.TransactionHalted:
-        pass
-    try:
-        # Either a complete success, or a conflict during push. But in
-        # either case, we've successfully effected the edits the user
-        # asked us for.
-        return trans.run(iw)
-    except transaction.TransactionException:
-        # Transaction aborted -- we couldn't check out files due to
-        # dirty index/worktree. The edits were not carried out.
-        return failed()
+    retval, _ = edit.perform_edit(
+        stack,
+        cd,
+        patchname,
+        new_patchname,
+        options.diff,
+        options.diff_flags,
+        options.set_tree,
+    )
+    return retval
