@@ -3,7 +3,13 @@ import re
 import sys
 
 from stgit.argparse import keep_option, opt, patch_range
-from stgit.commands.common import CmdException, DirectoryHasRepository, parse_patches
+from stgit.commands.common import (
+    CmdException,
+    DirectoryHasRepository,
+    check_head_top_equal,
+    check_index_and_worktree_clean,
+    parse_patches,
+)
 from stgit.lib import transaction
 
 __copyright__ = """
@@ -38,7 +44,7 @@ options = [
     opt(
         '--noapply',
         action='store_true',
-        short='Reorder patches by floating without applying.',
+        short='Reorder patches by floating without applying',
     ),
     opt(
         '-s',
@@ -80,13 +86,14 @@ def func(parser, options, args):
         raise CmdException('No patches to float')
 
     iw = stack.repository.default_iw
-    if options.keep or (
-        options.noapply and not any(pn in stack.patchorder.applied for pn in patches)
+    check_head_top_equal(stack)
+
+    if not options.keep and (
+        not options.noapply or any(pn in stack.patchorder.applied for pn in patches)
     ):
-        clean_iw = None
-    else:
-        clean_iw = iw
-    trans = transaction.StackTransaction(stack, 'float', check_clean_iw=clean_iw)
+        check_index_and_worktree_clean(stack)
+
+    trans = transaction.StackTransaction(stack)
 
     if options.noapply:
         applied = [p for p in trans.applied if p not in patches]
@@ -99,4 +106,4 @@ def func(parser, options, args):
         trans.reorder_patches(applied, unapplied, iw=iw)
     except transaction.TransactionHalted:
         pass
-    return trans.run(iw)
+    return trans.execute('float', iw)
