@@ -101,7 +101,7 @@ options = [
         short='Only fold the given file (can be used multiple times)',
     ),
     opt(
-        '--unapplied',
+        '--noapply',
         action='store_true',
         short='Keep the patch unapplied',
     ),
@@ -120,7 +120,8 @@ def __pick_commit(stack, ref_stack, iw, commit, patchname, options):
         patchname = 'revert-' + patchname
 
     if patchname:
-        patchname = stack.patches.make_name(patchname, lower=False)
+        if patchname in stack.patches:
+            patchname = stack.patches.make_name(patchname, lower=False)
     else:
         patchname = stack.patches.make_name(commit.data.message_str)
 
@@ -224,17 +225,24 @@ def __pick_commit(stack, ref_stack, iw, commit, patchname, options):
             )
         )
 
-        trans = StackTransaction(stack, 'pick %s from %s' % (patchname, ref_stack.name))
+        if not options.noapply:
+            check_head_top_equal(stack)
+
+        trans = StackTransaction(stack)
         trans.patches[patchname] = new_commit
 
         trans.unapplied.append(patchname)
-        if not options.unapplied:
+        if not options.noapply:
             try:
                 trans.push_patch(patchname, iw, allow_interactive=True)
             except TransactionHalted:
                 pass
 
-        retval = trans.run(iw, print_current_patch=False)
+        retval = trans.execute(
+            'pick %s from %s' % (patchname, ref_stack.name),
+            iw,
+            print_current_patch=False,
+        )
 
         if retval == STGIT_CONFLICT:
             out.done('conflict(s)')
@@ -258,7 +266,7 @@ def func(parser, options, args):
     stack = repository.get_stack()
     iw = repository.default_iw
 
-    if not options.unapplied:
+    if not options.noapply:
         check_local_changes(repository)
         check_conflicts(iw)
         check_head_top_equal(stack)
@@ -306,8 +314,6 @@ def func(parser, options, args):
         patchname = None
         retval = __pick_commit(stack, ref_stack, iw, commit, patchname, options)
     else:
-        if options.unapplied:
-            patches.reverse()
         for patchname in patches:
             commit = git_commit(patchname, repository, ref_stack.name)
             retval = __pick_commit(stack, ref_stack, iw, commit, patchname, options)
