@@ -1,5 +1,11 @@
 from stgit.argparse import keep_option, merged_option, opt, patch_range
-from stgit.commands.common import CmdException, DirectoryHasRepository, parse_patches
+from stgit.commands.common import (
+    CmdException,
+    DirectoryHasRepository,
+    check_head_top_equal,
+    check_index_and_worktree_clean,
+    parse_patches,
+)
 from stgit.lib import transaction
 
 __copyright__ = """
@@ -59,7 +65,7 @@ options = [
     opt(
         '--noapply',
         action='store_true',
-        short='Reorder patches by pushing without applying.',
+        short='Reorder patches by pushing without applying',
     ),
     opt(
         '--set-tree',
@@ -123,11 +129,11 @@ def func(parser, options, args):
 
     assert patches
 
-    if options.keep or options.noapply:
-        clean_iw = None
-    else:
-        clean_iw = iw
-    trans = transaction.StackTransaction(stack, 'push', check_clean_iw=clean_iw)
+    check_head_top_equal(stack)
+    if not options.keep and not options.noapply:
+        check_index_and_worktree_clean(stack)
+
+    trans = transaction.StackTransaction(stack)
 
     if options.reverse:
         patches.reverse()
@@ -154,4 +160,4 @@ def func(parser, options, args):
                 )
         except transaction.TransactionHalted:
             pass
-    return trans.run(iw)
+    return trans.execute('push', iw)
