@@ -3,6 +3,7 @@ from stgit.argparse import opt
 from stgit.commands.common import (
     CmdException,
     DirectoryHasRepository,
+    check_head_top_equal,
     run_commit_msg_hook,
 )
 from stgit.config import config
@@ -195,22 +196,21 @@ def write_tree(stack, paths, use_temp_index):
     :rtype: :class:`Tree<stgit.git.Tree>`
 
     """
-
-    def go(index):
-        if paths:
-            iw = IndexAndWorktree(index, stack.repository.default_worktree)
-            iw.update_index(paths)
-        return index.write_tree()
-
     if use_temp_index:
         with stack.repository.temp_index() as index:
+            iw = IndexAndWorktree(index, stack.repository.default_worktree)
             try:
                 index.read_tree(stack.head)
-                return go(index)
+                if paths:
+                    iw.update_index(paths)
+                return index.write_tree()
             finally:
                 stack.repository.default_iw.update_index(paths)
     else:
-        return go(stack.repository.default_index)
+        iw = stack.repository.default_iw
+        if paths:
+            iw.update_index(paths)
+        return iw.index.write_tree()
 
 
 def make_temp_patch(stack, patch_name, tree):
@@ -223,11 +223,15 @@ def make_temp_patch(stack, patch_name, tree):
         )
     )
     temp_name = stack.patches.make_name('refresh-temp')
-    trans = StackTransaction(stack, 'refresh (create temporary patch)')
+    trans = StackTransaction(stack)
     trans.patches[temp_name] = commit
     trans.applied.append(temp_name)
     return (
-        trans.run(stack.repository.default_iw, print_current_patch=False),
+        trans.execute(
+            'refresh (create temporary patch)',
+            stack.repository.default_iw,
+            print_current_patch=False,
+        ),
         temp_name,
     )
 
@@ -326,28 +330,22 @@ def absorb(stack, patch_name, temp_name, edit_fun, annotate=None):
     if annotate:
         log_msg += '\n\n' + annotate
 
-    trans = StackTransaction(stack, log_msg)
+    check_head_top_equal(stack)
+    trans = StackTransaction(stack)
     iw = stack.repository.default_iw
     if patch_name in trans.applied:
-        absorb_func = absorb_applied
+        absorbed = absorb_applied(trans, iw, patch_name, temp_name, edit_fun)
     else:
-        absorb_func = absorb_unapplied
+        absorbed = absorb_unapplied(trans, iw, patch_name, temp_name, edit_fun)
 
-    if absorb_func(trans, iw, patch_name, temp_name, edit_fun):
+    r = trans.execute(log_msg, iw)
 
-        def info_msg():
-            pass
+    if not absorbed:
+        out.warn(
+            'The new changes did not apply cleanly to %s.' % patch_name,
+            'They were saved in %s.' % temp_name,
+        )
 
-    else:
-
-        def info_msg():
-            out.warn(
-                'The new changes did not apply cleanly to %s.' % patch_name,
-                'They were saved in %s.' % temp_name,
-            )
-
-    r = trans.run(iw)
-    info_msg()
     return r
 
 
@@ -366,13 +364,14 @@ def __refresh_spill(annotate):
     if annotate:
         log_msg += '\n\n' + annotate
 
-    trans = StackTransaction(stack, log_msg, allow_conflicts=True)
+    check_head_top_equal(stack)
+    trans = StackTransaction(stack, allow_conflicts=True)
     trans.patches[patchname] = stack.repository.commit(cd)
     try:
         # Either a complete success, or a conflict during push. But in
         # either case, we've successfully effected the edits the user
         # asked us for.
-        return trans.run()
+        return trans.execute(log_msg)
     except TransactionException:
         # Transaction aborted -- we couldn't check out files due to
         # dirty index/worktree. The edits were not carried out.
@@ -428,6 +427,8 @@ def __refresh(
                 'To force a full refresh use --force.'
             )
 
+    check_head_top_equal(stack)
+
     # Update index and write tree
     tree = write_tree(stack, paths, use_temp_index=use_temp_index)
 
