@@ -3,6 +3,7 @@ from stgit.argparse import opt, patch_range
 from stgit.commands.common import (
     CmdException,
     DirectoryHasRepository,
+    check_head_top_equal,
     parse_patches,
     run_commit_msg_hook,
     strip_comments,
@@ -113,7 +114,9 @@ def squash(stack, patches, name=None, msg=None, no_verify=False):
 
     iw = stack.repository.default_iw
 
-    trans = StackTransaction(stack, 'squash', allow_conflicts=True)
+    check_head_top_equal(stack)
+
+    trans = StackTransaction(stack, allow_conflicts=True)
     push_new_patch = bool(set(patches) & set(trans.applied))
     new_patch_name = None
     try:
@@ -148,7 +151,7 @@ def squash(stack, patches, name=None, msg=None, no_verify=False):
             trans.push_patch(pn, iw)
     except TransactionHalted:
         pass
-    return trans.run(iw), new_patch_name
+    return trans.execute('squash', iw), new_patch_name
 
 
 def func(parser, options, args):
