@@ -3,7 +3,13 @@ import os
 import re
 
 from stgit.argparse import opt, patch_range
-from stgit.commands.common import CmdException, DirectoryGotoTopLevel, parse_patches
+from stgit.commands.common import (
+    CmdException,
+    DirectoryGotoTopLevel,
+    check_head_top_equal,
+    check_index_and_worktree_clean,
+    parse_patches,
+)
 from stgit.lib.git import CommitData
 from stgit.lib.transaction import StackTransaction, TransactionHalted
 from stgit.out import out
@@ -156,16 +162,18 @@ def func(parser, options, args):
         raise CmdException('No common patches to be synchronised')
 
     iw = repository.default_iw
+    check_head_top_equal(stack)
+    check_index_and_worktree_clean(stack)
 
     # pop to the one before the first patch to be synchronised
     first_patch = sync_patches[0]
     if first_patch in applied:
         to_pop = applied[applied.index(first_patch) + 1 :]
         if to_pop:
-            trans = StackTransaction(stack, 'sync (pop)', check_clean_iw=iw)
+            trans = StackTransaction(stack)
             popped_extra = trans.pop_patches(lambda pn: pn in to_pop)
             assert not popped_extra
-            retval = trans.run(iw)
+            retval = trans.execute('sync (pop)', iw)
             assert not retval
         pushed = [first_patch]
     else:
@@ -173,7 +181,7 @@ def func(parser, options, args):
         pushed = []
     popped = to_pop + [p for p in patches if p in unapplied]
 
-    trans = StackTransaction(stack, 'sync', check_clean_iw=iw)
+    trans = StackTransaction(stack)
     try:
         for p in pushed + popped:
             if p in popped:
@@ -197,4 +205,4 @@ def func(parser, options, args):
                 out.done()
     except TransactionHalted:
         pass
-    return trans.run(iw)
+    return trans.execute('sync', iw)
