@@ -153,11 +153,9 @@ def func(parser, options, args):
             taken_names.add(pn)
         patchnames.reverse()
 
-    trans = transaction.StackTransaction(
-        stack, 'uncommit', allow_conflicts=True, allow_bad_head=True
-    )
+    trans = transaction.StackTransaction(stack, allow_conflicts=True)
     for commit, pn in zip(commits, patchnames):
         trans.patches[pn] = commit
     trans.applied = list(reversed(patchnames)) + trans.applied
-    trans.run(set_head=False)
+    trans.execute('uncommit', set_head=False)
     out.done()
