@@ -1,5 +1,10 @@
 from stgit.argparse import opt, patch_range
-from stgit.commands.common import CmdException, DirectoryHasRepository, parse_patches
+from stgit.commands.common import (
+    CmdException,
+    DirectoryHasRepository,
+    check_head_top_equal,
+    parse_patches,
+)
 from stgit.lib import transaction
 
 __copyright__ = """
@@ -41,7 +46,8 @@ directory = DirectoryHasRepository()
 def func(parser, options, args):
     """Unhide a range of patch in the series."""
     stack = directory.repository.current_stack
-    trans = transaction.StackTransaction(stack, 'unhide')
+    check_head_top_equal(stack)
+    trans = transaction.StackTransaction(stack)
 
     if not args:
         parser.error('No patches specified')
@@ -56,4 +62,4 @@ def func(parser, options, args):
     hidden = [p for p in trans.hidden if p not in set(patches)]
 
     trans.reorder_patches(applied, unapplied, hidden)
-    return trans.run()
+    return trans.execute('unhide')
