@@ -16,6 +16,13 @@ class TransactionException(exception.StgException):
     """Exception raised when something goes wrong with a :class:`StackTransaction`."""
 
 
+class TransactionAborted(TransactionException):
+    """Raised when :class:`StackTransaction` aborts."""
+
+    def __init__(self):
+        super().__init__('Command aborted (all changes rolled back)')
+
+
 class TransactionHalted(TransactionException):
     """Exception raised when a :class:`StackTransaction` stops part-way through.
 
@@ -85,15 +92,7 @@ class StackTransaction:
 
     """
 
-    def __init__(
-        self,
-        stack,
-        msg,
-        discard_changes=False,
-        allow_conflicts=False,
-        allow_bad_head=False,
-        check_clean_iw=None,
-    ):
+    def __init__(self, stack, discard_changes=False, allow_conflicts=False):
         """Initialize a new :class:`StackTransaction`.
 
         :param discard_changes: Discard any changes in index+worktree
@@ -104,26 +103,24 @@ class StackTransaction:
 
         """
         self.stack = stack
-        self._msg = msg
+
         self.patches = _TransPatchMap(stack)
         self._applied = list(self.stack.patchorder.applied)
         self._unapplied = list(self.stack.patchorder.unapplied)
         self._hidden = list(self.stack.patchorder.hidden)
-        self._error = None
+        self._updated_head = None
+        self._updated_base = None
         self._current_tree = self.stack.head.data.tree
-        self._base = self.stack.base
-        self._discard_changes = discard_changes
-        self._bad_head = None
+        self._temp_index = None
+        self._temp_index_tree = None
+        self._error = None
         self._conflicts = None
+
+        self._discard_changes = discard_changes
         if isinstance(allow_conflicts, bool):
-            self._allow_conflicts = lambda trans: allow_conflicts
+            self._allow_conflicts = lambda _: allow_conflicts
         else:
             self._allow_conflicts = allow_conflicts
-        self._temp_index = self.temp_index_tree = None
-        if not allow_bad_head:
-            self._assert_head_top_equal()
-        if check_clean_iw:
-            self._assert_index_worktree_clean(check_clean_iw)
 
     @property
     def applied(self):
@@ -155,12 +152,15 @@ class StackTransaction:
 
     @property
     def base(self):
-        return self._base
+        if self._updated_base is None:
+            return self.stack.base
+        else:
+            return self._updated_base
 
     @base.setter
     def base(self, value):
         assert not self._applied or self.patches[self.applied[0]].data.parent == value
-        self._base = value
+        self._updated_base = value
 
     @property
     def temp_index(self):
@@ -174,57 +174,43 @@ class StackTransaction:
         if self._applied:
             return self.patches[self._applied[-1]]
         else:
-            return self._base
+            return self.base
 
     @property
     def head(self):
-        if self._bad_head:
-            return self._bad_head
+        if self._updated_head:
+            return self._updated_head
         else:
             return self.top
 
     @head.setter
     def head(self, value):
-        self._bad_head = value
+        self._updated_head = value
 
-    def _assert_head_top_equal(self):
-        if self.stack.head != self.stack.top:
+    def _checkout(self, tree, iw, allow_bad_head):
+        assert iw is not None
+        if not allow_bad_head and self.stack.head != self.stack.top:
             out.error(
                 'HEAD and top are not the same.',
                 'This can happen if you modify a branch with git.',
                 '"stg repair --help" explains more about what to do next.',
             )
-            self._abort()
-
-    def _assert_index_worktree_clean(self, iw):
-        if not iw.worktree_clean():
-            self._halt('Worktree not clean. Use "refresh" or "reset --hard"')
-        if not iw.index.is_clean(self.stack.head):
-            self._halt('Index not clean. Use "refresh" or "reset --hard"')
-
-    def _checkout(self, tree, iw, allow_bad_head):
-        if not allow_bad_head:
-            self._assert_head_top_equal()
+            raise TransactionAborted()
         if self._current_tree == tree and not self._discard_changes:
             # No tree change, but we still want to make sure that
             # there are no unresolved conflicts. Conflicts
             # conceptually "belong" to the topmost patch, and just
             # carrying them along to another patch is confusing.
-            if self._allow_conflicts(self) or iw is None or not iw.index.conflicts():
+            if self._allow_conflicts(self) or not iw.index.conflicts():
                 return
             out.error('Need to resolve conflicts first')
-            self._abort()
-        assert iw is not None
+            raise TransactionAborted()
         if self._discard_changes:
             iw.checkout_hard(tree)
         else:
             iw.checkout(self._current_tree, tree)
         self._current_tree = tree
 
-    @staticmethod
-    def _abort():
-        raise TransactionException('Command aborted (all changes rolled back)')
-
     def _check_consistency(self):
         remaining = set(self.all_patches)
         for pn, commit in self.patches.items():
@@ -233,13 +219,13 @@ class StackTransaction:
             else:
                 assert pn in remaining
 
-    def abort(self, iw=None):
-        # The only state we need to restore is index+worktree.
-        if iw:
-            self._checkout(self.stack.head.data.tree, iw, allow_bad_head=True)
-
-    def run(
-        self, iw=None, set_head=True, allow_bad_head=False, print_current_patch=True
+    def execute(
+        self,
+        msg,
+        iw=None,
+        set_head=True,
+        allow_bad_head=False,
+        print_current_patch=True,
     ):
         """Execute the transaction.
 
@@ -248,18 +234,18 @@ class StackTransaction:
         """
         self._check_consistency()
         log_external_mods(self.stack)
-        new_head = self.head
 
         # Set branch head.
         if set_head:
             if iw:
                 try:
-                    self._checkout(new_head.data.tree, iw, allow_bad_head)
+                    self._checkout(self.head.data.tree, iw, allow_bad_head)
                 except CheckoutException:
                     # We have to abort the transaction.
-                    self.abort(iw)
-                    self._abort()
-            self.stack.set_head(new_head, self._msg)
+                    # The only state we need to restore is index+worktree.
+                    self._checkout(self.stack.head.data.tree, iw, allow_bad_head=True)
+                    raise TransactionAborted()
+            self.stack.set_head(self.head, msg)
 
         if self._error:
             if self._conflicts:
@@ -268,7 +254,8 @@ class StackTransaction:
                 out.error(self._error)
 
         old_applied = self.stack.patchorder.applied
-        msg = self._msg + (' (CONFLICT)' if self._conflicts else '')
+        if self._conflicts:
+            msg = '%s (CONFLICT)' % (msg,)
 
         # Write patches.
         for pn, commit in self.patches.items():
@@ -323,6 +310,34 @@ class StackTransaction:
         self._print_popped(popped)
         return popped1
 
+    def hide_patches(self, p, quiet=False):
+        """Hide all patches pn for which p(pn) is true.
+
+        Always succeeds.
+
+        :returns: list of other patches that had to be popped to accomplish this.
+
+        """
+        popped = []
+        all_patches = self.applied + self.unapplied + self.hidden
+        for i in range(len(self.applied)):
+            if p(self.applied[i]):
+                popped = self.applied[i:]
+                del self.applied[i:]
+                break
+        popped = [pn for pn in popped if not p(pn)]
+        self._print_popped(popped)
+        self.unapplied = popped + [pn for pn in self.unapplied if not p(pn)]
+        new_hidden = []
+        for pn in all_patches:
+            if pn not in self.hidden and p(pn):
+                new_hidden.append(pn)
+                if not quiet:
+                    s = ['', ' (empty)'][self.patches[pn].data.is_nochange()]
+                    out.info('Hid %s%s' % (pn, s))
+        self.hidden = new_hidden + self.hidden
+        return popped
+
     def delete_patches(self, p, quiet=False):
         """Delete all patches pn for which p(pn) is true.
 
@@ -369,8 +384,8 @@ class StackTransaction:
             base = oldparent.data.tree
             ours = cd.parent.data.tree
             theirs = cd.tree
-            tree, self.temp_index_tree = self.temp_index.merge(
-                base, ours, theirs, self.temp_index_tree
+            tree, self._temp_index_tree = self.temp_index.merge(
+                base, ours, theirs, self._temp_index_tree
             )
         s = ''
         merge_conflict = False
@@ -507,10 +522,10 @@ class StackTransaction:
         merged = []
         if tree:
             self.temp_index.read_tree(tree)
-            self.temp_index_tree = tree
-        elif self.temp_index_tree != self.stack.head.data.tree:
+            self._temp_index_tree = tree
+        elif self._temp_index_tree != self.stack.head.data.tree:
             self.temp_index.read_tree(self.stack.head.data.tree)
-            self.temp_index_tree = self.stack.head.data.tree
+            self._temp_index_tree = self.stack.head.data.tree
         for pn in reversed(patches):
             # check whether patch changes can be reversed in the current index
             cd = self.patches[pn].data
@@ -525,7 +540,7 @@ class StackTransaction:
                 merged.append(pn)
                 # The self.temp_index was modified by apply_treediff() so
                 # force read_tree() the next time merge() is used.
-                self.temp_index_tree = None
+                self._temp_index_tree = None
             except MergeException:
                 pass
         if not quiet:
