@@ -5,11 +5,11 @@ test_description='Test the push and pop commands'
 
 test_expect_success \
     'Test behavior on uninitialized repo' '
-    command_error stg prev 2>&1 | grep -e "branch not initialized" &&
-    command_error stg next 2>&1 | grep -e "branch not initialized" &&
-    command_error stg top 2>&1 | grep -e "branch not initialized" &&
-    command_error stg pop 2>&1 | grep -e "branch not initialized" &&
-    command_error stg push 2>&1 | grep -e "branch not initialized"
+    command_error stg prev 2>err && grep -e "branch not initialized" err &&
+    command_error stg next 2>err && grep -e "branch not initialized" err &&
+    command_error stg top 2>err && grep -e "branch not initialized" err &&
+    command_error stg pop 2>err && grep -e "branch not initialized" err &&
+    command_error stg push 2>err && grep -e "branch not initialized" err
 '
 test_expect_success \
     'Initialize the StGit repository' \
@@ -17,11 +17,11 @@ test_expect_success \
 
 test_expect_success \
     'Test behavior on empty repo' '
-    command_error stg prev 2>&1 | grep -e "Not enough applied patches" &&
-    command_error stg next 2>&1 | grep -e "No unapplied patches" &&
-    command_error stg top  2>&1 | grep -e "No patches applied" &&
-    command_error stg pop  2>&1 | grep -e "No patches applied" &&
-    command_error stg push 2>&1 | grep -e "No patches to push"
+    command_error stg prev 2>err && grep -e "Not enough applied patches" err &&
+    command_error stg next 2>err && grep -e "No unapplied patches" err &&
+    command_error stg top  2>err && grep -e "No patches applied" err &&
+    command_error stg pop  2>err && grep -e "No patches applied" err &&
+    command_error stg push 2>err && grep -e "No patches to push" err
 '
 
 test_expect_success \
@@ -36,7 +36,7 @@ test_expect_success \
 
 test_expect_success \
     'Check prev, next, and top with all applied' '
-    command_error stg next 2>&1 | grep -e "No unapplied patches" &&
+    command_error stg next 2>err && grep -e "No unapplied patches" err &&
     [ "$(echo $(stg prev))" = "p8" ] &&
     [ "$(echo $(stg top))" = "p9" ]
 '
@@ -56,9 +56,9 @@ test_expect_success \
 
 test_expect_success \
     'Check prev, next, and top with invalid arguments' '
-    command_error stg prev bogus_arg 2>&1 | grep -e "incorrect number of arguments" &&
-    command_error stg next bogus_arg 2>&1 | grep -e "incorrect number of arguments" &&
-    command_error stg top  bogus_arg 2>&1 | grep -e "incorrect number of arguments"
+    command_error stg prev bogus_arg 2>err && grep -e "incorrect number of arguments" err &&
+    command_error stg next bogus_arg 2>err && grep -e "incorrect number of arguments" err &&
+    command_error stg top  bogus_arg 2>err && grep -e "incorrect number of arguments" err
 '
 
 test_expect_success \
@@ -66,7 +66,8 @@ test_expect_success \
     stg pop -a &&
     [ "$(echo $(stg series --applied --noprefix))" = "" ] &&
     [ "$(echo $(stg series --unapplied --noprefix))" = "p0 p1 p2 p3 p4 p5 p6 p7 p8 p9" ] &&
-    command_error stg pop 2>&1 | grep -e "No patches applied"
+    command_error stg pop 2>err &&
+    grep -e "No patches applied" err
 '
 
 test_expect_success \
@@ -81,7 +82,8 @@ test_expect_success \
     stg push -a &&
     [ "$(echo $(stg series --applied --noprefix))" = "p0 p1 p2 p3 p4 p5 p6 p7 p8 p9" ] &&
     [ "$(echo $(stg series --unapplied --noprefix))" = "" ] &&
-    command_error stg push 2>&1 | grep -e "No patches to push"
+    command_error stg push 2>err &&
+    grep -e "No patches to push" err
 '
 
 test_expect_success \
@@ -139,8 +141,8 @@ test_expect_success \
 
 test_expect_success \
     'Attempt to push already applied patches' '
-    command_error stg push p0..p2 2>&1 | grep -e "Patches already applied" &&
-    command_error stg push p99999 2>&1 | grep -e "Unknown patch name: p99999"
+    command_error stg push p0..p2 2>err && grep -e "Patches already applied" err &&
+    command_error stg push p99999 2>err && grep -e "Unknown patch name: p99999" err
 '
 
 test_done
