@@ -29,7 +29,8 @@ test_expect_success 'Initialize StGit stack' '
 '
 
 test_expect_success 'sink default without applied patches' '
-    command_error stg sink
+    command_error stg sink 2>err &&
+    grep -e "No patches to sink" err
 '
 
 test_expect_success 'sink and reorder specified without applied patches' '
@@ -37,6 +38,11 @@ test_expect_success 'sink and reorder specified without applied patches' '
     test "$(echo $(stg series --applied --noprefix))" = "p2 p1"
 '
 
+test_expect_success 'attempt sink below unapplied' '
+    command_error stg sink --to=p4 2>err &&
+    grep -e "Cannot sink below p4 since it is not applied" err
+'
+
 test_expect_success 'sink patches to the bottom of the stack' '
     stg sink p4 p3 p2 &&
     test "$(echo $(stg series --applied --noprefix))" = "p4 p3 p2 p1"
@@ -57,6 +63,25 @@ test_expect_success 'sink specified patch below a target' '
     test "$(echo $(stg series --applied --noprefix))" = "p1 p2 p3 p4"
 '
 
+test_expect_success 'sink --nopush' '
+    stg sink --nopush --to=p2 &&
+    test "$(echo $(stg series --applied --noprefix))" = "p1 p4" &&
+    test "$(echo $(stg series --unapplied --noprefix))" = "p2 p3 p22"
+'
+
+test_expect_success 'sink --nopush with multiple patches' '
+    stg sink --nopush p1 p2 p3 &&
+    test "$(echo $(stg series --applied --noprefix))" = "p1 p2 p3" &&
+    test "$(echo $(stg series --unapplied --noprefix))" = "p4 p22" &&
+    stg goto p4
+'
+
+test_expect_success 'attempt sink with same to and target' '
+    command_error stg sink --to=p3 p3 2>err &&
+    grep -e "Cannot have a sinked patch as target" err &&
+    rm err
+'
+
 test_expect_success 'sink with conflict' '
     conflict stg sink --to=p2 p22 &&
     test "$(echo $(stg series --applied --noprefix))" = "p1 p22" &&
