@@ -16,7 +16,7 @@ test_expect_success \
 test_expect_success \
     'Apply a patch from an 8bit-encoded e-mail' \
     '
-    stg import -m "$TEST_DIRECTORY"/t1801/email-8bit &&
+    stg import -m --message-id "$TEST_DIRECTORY"/t1801/email-8bit &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
@@ -35,7 +35,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
@@ -49,7 +49,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
@@ -67,7 +67,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
@@ -81,7 +81,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
@@ -94,7 +94,7 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
@@ -107,14 +107,15 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id) \
-        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
 test_expect_success \
     'Apply several patches from an mbox file' \
     '
-    stg import -M "$TEST_DIRECTORY"/t1801/email-mbox &&
+    test_config stgit.import.message-id "no" &&
+    stg import -M --message-id "$TEST_DIRECTORY"/t1801/email-mbox &&
     [ $(git cat-file -p $(stg id change-1) \
         | grep -c "tree 401bef82cd9fb403aba18f480a63844416a2e023") = 1 ] &&
     [ $(git cat-file -p $(stg id change-1) \
@@ -139,6 +140,7 @@ test_expect_success \
 test_expect_success \
     'Apply several patches from an mbox url' \
     '
+    test_config stgit.import.message-id "yes" &&
     stg import -u -M "file://$TEST_DIRECTORY"/t1801/email-mbox &&
     [ $(git cat-file -p $(stg id change-1) \
         | grep -c "tree 401bef82cd9fb403aba18f480a63844416a2e023") = 1 ] &&
@@ -172,25 +174,26 @@ test_expect_success \
 test_expect_success \
     'Apply several patches from an mbox file from stdin' \
     '
+    test_config stgit.import.message-id "off" &&
     cat "$TEST_DIRECTORY"/t1801/email-mbox | stg import -M &&
     [ $(git cat-file -p $(stg id change-1) \
         | grep -c "tree 401bef82cd9fb403aba18f480a63844416a2e023") = 1 ] &&
     [ $(git cat-file -p $(stg id change-1) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id change-1) \
-        | grep -c "Message-Id: <20061111114527.31778.12942.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111114527.31778.12942.stgit@localhost>") = 0 ] &&
     [ $(git cat-file -p $(stg id change-2) \
         | grep -c "tree e49dbce010ec7f441015a8c64bce0b99108af4cc") = 1 ] &&
     [ $(git cat-file -p $(stg id change-2) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id change-2) \
-        | grep -c "Message-Id: <20061111114527.31778.92851.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111114527.31778.92851.stgit@localhost>") = 0 ] &&
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "tree 166bbaf27a44aee21ba78c98822a741e6f7d78f5") = 1 ] &&
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     [ $(git cat-file -p $(stg id change-3-colon) \
-        | grep -c "Message-Id: <20061111114527.31778.45876.stgit@localhost>") = 1 ] &&
+        | grep -c "Message-Id: <20061111114527.31778.45876.stgit@localhost>") = 0 ] &&
     stg delete ..
     '
 
