@@ -22,15 +22,15 @@ test_expect_success \
 test_expect_success \
     'Test invalid remote argument' \
     '
-    command_error stg pull origin 2>&1 | \
-        grep "specifying a repository is meaningless for policy=\"rebase\""
+    command_error stg pull origin 2>err &&
+    grep "specifying a repository is meaningless for policy=\"rebase\"" err
     '
 
 test_expect_success \
     'Test invalid arguments' \
     '
-    command_error stg pull origin master 2>&1 | \
-        grep "incorrect number of arguments"
+    command_error stg pull origin master 2>err &&
+    grep "incorrect number of arguments" err
     '
 
 test_expect_success \
