@@ -14,21 +14,26 @@ test_expect_success 'Setup upstream with patches' '
     )
 '
 
-test_expect_success 'Pull master and stack with all applied' '
+test_expect_success 'Fetch and reset master and stack with all applied' '
     test_create_repo cloned &&
     (cd cloned &&
      git config pull.ff only &&
-     git pull -f ../upstream master:master refs/stacks/master:refs/stacks/master &&
+     git remote add upstream ../upstream &&
+     git fetch upstream master &&
+     git reset --hard upstream/master &&
+     git fetch -f upstream refs/stacks/master:refs/stacks/master &&
      [ "$(echo $(stg series --applied --noprefix))" = "patch-1 patch-2 patch-3" ]
     )
 '
 
-test_expect_success 'Pull master and stack with unapplied patches' '
+test_expect_success 'Fetch and reset master and stack with unapplied patches' '
     (cd upstream &&
      stg pop
     ) &&
     (cd cloned &&
-     git pull -f ../upstream master:master refs/stacks/master:refs/stacks/master &&
+     git fetch upstream master &&
+     git reset --hard upstream/master &&
+     git fetch -f upstream refs/stacks/master:refs/stacks/master &&
      [ "$(echo $(stg series --applied --noprefix))" = "patch-1 patch-2" ] &&
      [ "$(echo $(stg series --unapplied --noprefix))" = "patch-3" ]
     )
