@@ -66,15 +66,15 @@ test_expect_success 'refresh --no-verify with path limiting, failing hook' '
 
 test_expect_success 'refresh with failing hook' '
     echo "pre-commit-hook-fail" >> file &&
-    command_error stg refresh 2>&1 |
-    grep -e "pre-commit hook failed" &&
+    command_error stg refresh 2>err &&
+    grep -e "pre-commit hook failed" err &&
     git reset HEAD
 '
 
 test_expect_success 'refresh with path limiting, failing hook' '
     echo "pre-commit-hook-path-limiting-fail" >> file &&
-    command_error stg refresh file 2>&1 |
-    grep -e "pre-commit hook failed" &&
+    command_error stg refresh file 2>err &&
+    grep -e "pre-commit hook failed" err &&
     git reset HEAD
 '
 
@@ -125,8 +125,8 @@ test_expect_success 'refresh with path limiting, succeeding hook, does not modif
 
 test_expect_success 'refresh with failing hook that modifies file' '
     echo "pre-commit-hook-remove-whitespace  " >> file &&
-    command_error stg refresh 2>&1 |
-    grep -e "pre-commit hook failed" &&
+    command_error stg refresh 2>err &&
+    grep -e "pre-commit hook failed" err &&
     [ "$(git diff --name-only)" = "file" ] &&
     [ "$(git diff --cached --name-only)" = "file" ] &&
     [ "$(tail -1 file)" = "pre-commit-hook-remove-whitespace" ]
