@@ -35,8 +35,8 @@ write_script diffedit <<EOF
 echo "" > "\$1"
 EOF
 test_expect_success 'Empty editor aborts edit' '
-    EDITOR=./diffedit command_error stg edit 2>&1 |
-    grep -e "Aborting edit due to empty patch description"
+    EDITOR=./diffedit command_error stg edit 2>err &&
+    grep -e "Aborting edit due to empty patch description" err
 '
 rm -f diffedit
 
