@@ -25,27 +25,27 @@ test_expect_success 'Initialize StGit repository' '
 '
 
 test_expect_success 'Attempt fold more than one patch' '
-    command_error stg fold fold1.diff fold2.diff 2>&1 |
-    grep -e "incorrect number of arguments"
+    command_error stg fold fold1.diff fold2.diff 2>err &&
+    grep -e "incorrect number of arguments" err
 '
 
 test_expect_success 'Attempt fold with local changes' '
     echo "hello dirty" > foo.txt &&
     test_when_finished "stg reset --hard" &&
-    command_error stg fold fold1.diff 2>&1 |
-    grep -e "local changes in the tree"
+    command_error stg fold fold1.diff 2>err &&
+    grep -e "local changes in the tree" err
 '
 
 test_expect_success 'Attempt fold with non-existant patch file' '
-    command_error stg fold non-existant.diff 2>&1 |
-    grep "No such file"
+    command_error stg fold non-existant.diff 2>err &&
+    grep "No such file" err
 '
 
 test_expect_success 'Attempt fold with no applied patches' '
     stg pop -a &&
     test_when_finished "stg push -a" &&
-    command_error stg fold fold1.diff 2>&1 |
-    grep "No patches applied"
+    command_error stg fold fold1.diff 2>err &&
+    grep "No patches applied" err
 '
 
 test_expect_success 'Fold a patch file' '
@@ -75,8 +75,8 @@ test_expect_success 'Attempt to fold conflicting patch' '
     echo "hello" > foo.txt &&
     echo "from p2" >> foo.txt &&
     stg refresh &&
-    command_error stg fold fold1.diff 2>&1 |
-    grep "Patch does not apply cleanly" &&
+    command_error stg fold fold1.diff 2>err &&
+    grep "Patch does not apply cleanly" err &&
     test -z "$(echo $(stg status --porcelain foo.txt))"
     test ! -e foo.txt.rej
 '
@@ -86,8 +86,8 @@ test_expect_success 'Attempt to fold conflicting patch with rejects' '
     echo "hello" > foo.txt &&
     echo "from p2" >> foo.txt &&
     stg refresh &&
-    command_error stg fold --reject fold1.diff 2>&1 |
-    grep "Patch does not apply cleanly" &&
+    command_error stg fold --reject fold1.diff 2>err &&
+    grep "Patch does not apply cleanly" err &&
     test -z "$(echo $(stg status --porcelain foo.txt))" &&
     test -e foo.txt.rej &&
     rm foo.txt.rej
